;; general ellipsise any header ?


;;; rss2leafnode.el --- misc helpers for rss2leafnode

;; Copyright 2012, 2013, 2014, 2017, 2019 Kevin Ryde
;;
;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: news, gnus, rss2leafnode
;; URL: http://user42.tuxfamily.org/rss2leafnode/index.html

;; rss2leafnode.el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; rss2leafnode.el is distributed in the hope that it will be
;; useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.

;;; Commentary:
;; 

;;; History:

;; Version 1 - the first version

;;; Code:

;;;###autoload
(defun rss2leafnode-gnus-article-wash-long-keywords ()
  "Shorten long Keywords: header in a Gnus article.
This is designed for use from `gnus-part-display-hook' but can
also be used interactively for one-off washing.

If there's a \"Keywords:\" header in the article then it's
shortened to at most 2 lines.  This is good for some rss2leafnode
articles where the originating RSS feed has a long list of
semi-automatically generated keywords.

The ellipsis is a button which can be pressed to show the whole
original.  \(There's no re-ellipsise, other than `\\<gnus-summary-mode-map>\\[gnus-summary-show-article]' show article
again.)"

  (interactive)
  (save-restriction
    (article-narrow-to-head)
    (save-excursion
      (goto-char (point-min))
      (while (re-search-forward "^\\Keywords:\\([ \t].*\n\\)+"
                                (point-max) t)
        (narrow-to-region (match-beginning 0) (1- (match-end 0)))
        (goto-char (point-min))
        (forward-line 1)
        (move-to-column (- (frame-width) 6))
        (unless (= (point) (point-max))
          (let ((str (buffer-substring (point) (point-max))))
            (delete-region (point) (point-max))
            (insert-text-button " ..."
                                'action       'rss2leafnode-gnus-article-wash-long-keywords-expand
                                'rss2leafnode str)))))))

(defun rss2leafnode-gnus-article-wash-long-keywordsn-expand (button)
  "An internal part of rss2leafnode.el.
This is the action function for the ellipsis button
of `rss2leafnode-gnus-article-wash-long-keywords'."
  (save-excursion
    (goto-char (button-start button))
    (let ((str (button-get button 'rss2leafnode)) ;; before deleting the button text
          (inhibit-read-only t))
      (delete-region (point) (button-end button))
      (insert str))))

(defun rss2leafnode-gnus-article-wash-face-line ()
  "Shorten any Face: or X-Face: header to a single line in a Gnus article.
This is designed for use from `gnus-part-display-hook' but can
also be used interactively.

Any \"Face:\" or \"X-Face:\" header in the article is shortened
to at most 1 line each.  This is good on a text-only display to
show that those headers are present, but not waste a lot of space
on their encoded data."

  (save-restriction
    (article-narrow-to-head)
    (save-excursion
      (goto-char (point-min))
      (while (re-search-forward "^\\(X-\\)?Face:[ \t].*\n\\([ \t].*\n\\)*"
                                (point-max) t)
        (goto-char (match-beginning 0))
        (move-to-column (- (frame-width) 5))
        (delete-region (point) (1- (match-end 0)))
        (insert " ...")))))

;;;###autoload
(custom-add-option 'gnus-part-display-hook 'rss2leafnode-gnus-article-wash-long-keywords)

;;;###autoload
(custom-add-option 'gnus-part-display-hook 'rss2leafnode-gnus-article-wash-face-line)

(provide 'rss2leafnode)

;;; rss2leafnode.el ends here
