;; Copyright 2009, 2010, 2013, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.



;;-----------------------------------------------------------------------------
;; overlays on nobreaks

(defun my-show-nobreak ()
  (interactive)
  (require 'texinfo-warn)
  (require 'man-completion)

  (save-excursion
    (dolist (overlay (overlays-in (point-min) (point-max)))
      (if (overlay-get overlay 'my-show-nobreak)
          (delete-overlay overlay)))

    (let* ((pair (bounds-of-thing-at-point 'paragraph))
           (end-para  (cdr pair)))
      (goto-char (car pair))

      (while (re-search-forward "[ \t]+" end-para t)
        (let ((beg-white (match-beginning 0))
              (end-white (match-end 0))
              pred-lst
              lst)
          (dolist (pred (man-completion-hook-to-list 'fill-nobreak-predicate))
            (goto-char end-white)
            (when (and (not (member pred lst))
                       (funcall pred))
              (push pred lst))
            (unless (= (point) end-white)
              (message "%S moved point" pred)))
          (goto-char end-white)

          (when lst
            (let ((overlay (make-overlay beg-white end-white
                                         (current-buffer) nil nil)))
              (overlay-put overlay 'my-show-nobreak t)
              (overlay-put overlay 'face 'texinfo-warn)
              (overlay-put overlay 'help-echo (mapconcat 'symbol-name lst "\n")))))))))

;; Some text a blah a blah.  A fjdsk fjksd fjksd.

;;-------------------------------------------------------------------------
;; my-nobreak-fade-tex-math-start-p

(let ((count 0))
  (switch-to-buffer "*t-nobreak-fade--temp*")
  (mapcar (lambda (elem)
            (let* ((str  (nth 0 elem))
                   (want (nth 1 elem))
                   (case-fold-search t))
              (erase-buffer)
              (insert str)
              (search-backward "|")
              (let ((got (not (not (my-nobreak-fade-tex-math-start-p)))))
                (unless (equal want got)
                  (error "str %S got %S want %S" str got want))
                (setq count (1+ count)))))

          ;; string, wanted-return
          '(
            ;; $ pair $
            ("$x |times y$"  t)
            ("blah   $x |times y$"  t)
            ("$x\t\t   |times y$"   t)
            ("$x$ |times "   nil)
            ("blah  $x$ |times "   nil)

            ;; $ not end by \$
            ("$10\\$ |times x$"  t)

            ;; $ other \x
            ("$10\\@ |times x$"  t)
            ("$\\$\\$ |times x$"  t)

            ;; $ end by \)
            ("$x\\) |times "   nil)

            ;; \( pair \)
            ("\\(x\\) |times "   nil)
            ("\\(x |times\\) "   t)

            ;; \( end by $
            ("\\(x$ |times "   nil)
            ("\\(x |times$ "   t)

            ;; \[ end by \]
            ("\\[x\\] |times "   nil)
            ("\\[x |times\\] "   t)

            ;; \[ not end by $ \], though doing that is invalid TeX
            ("\\[x$ |times \]"   t)
            ("\\[x\$ |times \]"   t)

            ;; ending $ allow break
            ("foo$ |blah"   nil)
            ("foo$$ |blah"   nil)
            ("$x foo$ |blah"   nil)
            ("$$x foo$$ |blah"   nil)
            ("foo\) |blah"   nil)
            ("\(x foo\) |blah"   nil)
            ("\[x foo\] |blah"   nil)
            ))

  (switch-to-buffer "d-nobreak-fade.el"))


;;-----------------------------------------------------------------------------

(find-file "d-nobreak-fade.txt")
(setq fill-nobreak-predicate
      '(fill-single-word-nobreak-p))

(setq fill-nobreak-predicate
      '(nobreak-fade-single-letter-p))

(add-hook    'fill-nobreak-predicate 'fill-french-nobreak-p)
(remove-hook 'fill-nobreak-predicate 'fill-french-nobreak-p)


;;-----------------------------------------------------------------------------
;; nobreak-fade-emacs-info-link-p

;; xxdsjkf sdjk fksjd fksd x fksdjf dskjf See Info
;; node `(emacs)Docum entation'



;;-----------------------------------------------------------------------------
;; nobreak-fade-sgml-wholetag-p

(add-hook 'fill-nobreak-predicate 'my-nobreak-fade-sgml-wholetag-p)
(remove-hook 'fill-nobreak-predicate 'my-nobreak-fade-sgml-wholetag-p)

<a type=fjkds/fjdskf href="fjkdsfdsjkfjsdkfdjkjfksdjfksdjfkjsdkfjsdkfjskdfjkdjfksdjfkdsjfkdjskfjsdk>


;;-----------------------------------------------------------------------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))
(progn
  (elint-initialize)
  (find-file "../nobreak-fade.el")
  (elint-current-buffer))

;;-----------------------------------------------------------------------------

(progn
  (defun foo ()
    "Blah.
Info node `(emacs)Top'
Info node `(emacs)Scroll
Calendar'
Info node `(emacs)Scroll Calendar'

Info node `(elisp)Documentation
Tips'

Info node `(elisp)Documentation Tips'

")
  (describe-function 'foo)
  (other-window 1))



(defvar nobreak-fade-history nil)

(nobreak-fade-completing-read '(nobreak-fade-emacs-M-x-p) nil)

(turn-on-completing-help-mode)


(defun nobreak-fade-functions ()
  "Return the list of functions in `fill-nobreak-predicate'.
This is an internal part of nobreak-fade.el."

  (cond ((not (or (eval-when-compile (boundp 'fill-nobreak-predicate))
                  (boundp 'fill-nobreak-predicate)))
         ;; xemacs21 -- no such feature at all, but recheck at runtime in
         ;; case an add-on creates something compatible
         )

        ((symbolp fill-nobreak-predicate)
         ;; single func value, probably emacs20,21, but can also have this
         ;; in emacs22
         (list fill-nobreak-predicate))

        ((or (eval-when-compile (get 'fill-nobreak-predicate 'custom-type))
             (get 'fill-nobreak-predicate 'custom-type))
         ;; emacs22 up -- a hook, remove buffer-local setting
         (remq t fill-nobreak-predicate))

        ;; emacs21 -- fill-nobreak-predicate is a variable holding a function
        ;; (lambda () (or ...))
        ((and (listp fill-nobreak-predicate)
              (eq 'lambda (car-safe fill-nobreak-predicate))
              (= 3 (safe-length fill-nobreak-predicate))
              (eq 'or (car-safe (nth 2 fill-nobreak-predicate))))
         (cdr (nth 2 fill-nobreak-predicate)))))
(nobreak-fade-functions)
  


(defun nobreak-fade-remove (pred &optional local)
  "Remove `texinfo-nobreak-p' from `fill-nobreak-predicate'."
  (interactive
   (progn
     (eval-and-compile
       (require 'cl))
     (list (nobreak-fade-completing-read
            (remove-if-not 'symbolp (nobreak-fade-functions))
            t))))

  (cond ((not (or (eval-when-compile (boundp 'fill-nobreak-predicate))
                  (boundp 'fill-nobreak-predicate)))
         ;; xemacs21 -- no such feature at all, but recheck at runtime in
         ;; case an add-on creates something compatible
         )

        ((or (eval-when-compile (get 'fill-nobreak-predicate 'custom-type))
             (get 'fill-nobreak-predicate 'custom-type))
         ;; emacs22 up -- a hook, remove buffer-local setting
         (remove-hook 'fill-nobreak-predicate pred local))

        ;; emacs21 -- fill-nobreak-predicate is a variable holding a function
        (t
         (let ((val (if local
                        (progn
                          (make-local-variable 'fill-nobreak-predicate)
                          fill-nobreak-predicate)
                      (default-value 'fill-nobreak-predicate))))
           (if (equal val pred)
               (setq val nil)
             ;; remove from (lambda () (or ...))
             (let (lst)
               (debug)
               (when (and (listp fill-nobreak-predicate)
                          (eq 'lambda (car-safe fill-nobreak-predicate))
                          (= 3 (safe-length fill-nobreak-predicate))
                          (eq 'or (car-safe
                                   (setq lst (nth 2 fill-nobreak-predicate)))))
                 (setq lst (remove (list pred)
                                   (remove pred (cdr lst))))
                 (setq val (and lst
                                `(lambda () (or ,@lst)))))))
           (if local
               (setq fill-nobreak-predicate val)
             (set-default fill-nobreak-predicate val))))))


(setq fill-nobreak-predicate 'nobreak-fade-emacs-info-link-p)
(setq fill-nobreak-predicate (lambda ()
                               (or (nobreak-fade-emacs-info-link-p))))
(setq fill-nobreak-predicate '(nobreak-fade-emacs-info-link-p))
(progn
  ;; (nobreak-fade-remove 'djfks)
  (nobreak-fade-remove 'nobreak-fade-emacs-info-link-p)
  fill-nobreak-predicate)
