;; Copyright 2011, 2014 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-elisp-docstring-preview--temp*"))
    (switch-to-buffer tempbuf)
    (emacs-lisp-mode)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(elisp-docstring-preview))


    ;;-------------------------------------------------------------------------
    ;; elisp-docstring-form-at-point

    (require 'elisp-docstring-preview)
    (mapcar (lambda (elem)
              (let ((str (car elem))
                    (want (cadr elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (let ((got (elisp-docstring-form-at-point)))
                  (unless (equal got want)
                    (error "elisp-docstring-form-at-point\ngot  %S\nwant %S"
                           got want)))
                (setq count (1+ count))))

            '(
              ("(defun foo (args) \"docstring\" 123)"
               (defun foo (args) "docstring" 123))

              ("(eval-when-compile (defun foo (args) \"docstring\" 123))"
               (defun foo (args) "docstring" 123))

              ("(if something (defun foo (args) \"docstring\" 123)
                              (defun bar () \"zzz\" 123))"
               (defun foo (args) "docstring" 123))
              ("(when something (defun foo (args) \"docstring\" 123))"
               (defun foo (args) "docstring" 123))
              ("(unless something (defun foo (args) \"docstring\" 123))"
               (defun foo (args) "docstring" 123))
              ("(static-if something (defun foo (args) \"docstring\" 123))"
               (defun foo (args) "docstring" 123))
              ("(static-when something (defun foo (args) \"docstring\" 123))"
               (defun foo (args) "docstring" 123))
              ("(static-unless something (defun foo (args) \"docstring\" 123))"
               (defun foo (args) "docstring" 123))
              ))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'elisp-docstring-preview)
    (unload-feature 'elisp-docstring-preview t)
    (require 'elisp-docstring-preview)
    (unload-feature 'elisp-docstring-preview t)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
