;; Copyright 2007, 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (let ((count 0)
        (tempbuf "*t-ffap-mml--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'ffap-mml)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (mapcar (lambda (elem)
              (let ((want-thing (nth 0 elem))
                    (want-ffap  (nth 1 elem))
                    (str        (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (while (not (eobp))
                  (let ((got-thing (thing-at-point 'ffap-mml-filename)))
                    (unless (string-equal want-thing got-thing)
                      (error "thing-at-point got %S want %S"
                             got-thing want-thing)))

                  (require 'ffap)
                  (setq ffap-string-at-point "dummy value")
                  (let ((got-ffap (ffap-string-at-point 'file)))
                    (unless (string-equal want-ffap got-ffap)
                      (error "ffap got %S want %S" got want))
                    (unless (eq got-ffap ffap-string-at-point)
                      (error "ffap got %S not put into ffap-string-at-point" got)))

                  (forward-char))

                (goto-char (point-max))
                (insert "\n")
                (let ((got-thing (thing-at-point 'ffap-mml-filename)))
                  (when got-thing
                    (error "thing-at-point after newline got %S want nil"
                           got-thing)))

                (setq count (1+ count))))

            '(("\"t-ffap-mml.el\""
               "t-ffap-mml.el"
               "<#part type=\"application/octet-stream\" filename=\"t-ffap-mml.el\" disposition=attachment>\n<#/part>")

              ;; unquoted
              ("something"
               "something"
               "<#part filename=something>\n<#/part>")
              ("something"
               "something"
               "<#part filename=something>")

              ;; escape \"
              ("\"/tmp/\\\"\""
               "/tmp/\""
               "<#part type=\"application/octet-stream\" filename=\"/tmp/\\\"\" disposition=attachment>\n<#/part>")

              ))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'ffap-mml)
            (prop    '(ffap-mml-filename bounds-of-thing-at-point)))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))
        (unless (apply 'get prop)
          (error "oops, didn't find thing-at-point property"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))

        (let ((func (apply 'get prop)))
          (when func
            (error "unload-feature leftover %S property: %S" prop func)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'ffap-mml))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
