;; Copyright 2007, 2008, 2009, 2010, 2011, 2012, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (case-fold-search t)
        (tempbuf "*t-perl-pod-gt.el temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(perl-pod-gt-single
              perl-pod-gt-double
              perl-pod-gt-nobreak-p))


    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((perl-mode-hook  perl-pod-gt-enable)
              (cperl-mode-hook perl-pod-gt-enable)
              (pod-mode-hook   perl-pod-gt-enable)))


    ;;-------------------------------------------------------------------------
    ;; perl-pod-gt-insert

    (overwrite-mode 0)
    (mapcar (lambda (elem)
              (let ((str  (car elem))
                    (want (cadr elem)))
                (erase-buffer)
                (insert str)
                (perl-pod-gt-insert)
                (let ((got (buffer-string)))
                  (unless (equal got want)
                    (error "perl-pod-gt-insert: got %s want %s" got want))))
              (setq count (1+ count)))

            '(("C<EE-"   "C<EE-E<gt>")
              ("c<EE-"   "c<EE->")
              ("c<"      "c<>")
              ("C<"      "C<>")
              ("c<x-"    "c<x->")
              ("C<x-"    "C<x-E<gt>")
              ("C<x="    "C<x=E<gt>")
              ("C<x "    "C<x E<gt>")
              ("C<x-E<-" "C<x-E<->")
              ("C<x> ="  "C<x> =>")
              ("C<x> C<E<gt>> -" "C<x> C<E<gt>> ->")
              ("C<x> C<E<gt> -"  "C<x> C<E<gt> -E<gt>")
              ("C<x> C<E<gt>E<gt> -"  "C<x> C<E<gt>E<gt> -E<gt>")
              ("C<< x-"  "C<< x->")
              ("C<<< x-" "C<<< x->")
              ("C<< E<gt> -"  "C<< E<gt> ->")
              ("C<< C<foo-"  "C<< C<foo->")))

    (overwrite-mode 1)
    (erase-buffer)
    (insert "C<EE-")
    (save-excursion (insert "xxxxxxxx"))
    (perl-pod-gt-insert)
    (let ((got (buffer-string))
          (want "C<EE-E<gt>xxx"))
      (unless (equal got want)
        (error "perl-pod-gt-insert overwrite: got %S want %S" got want)))
    (setq count (1+ count))

    (erase-buffer)
    (insert "blah")
    (save-excursion (insert "blah"))
    (perl-pod-gt-insert)
    (let ((got (buffer-string))
          (want "blah>lah"))
      (unless (equal got want)
        (error "perl-pod-gt-insert overwrite: got %S want %S" got want)))
    (setq count (1+ count))


    (overwrite-mode 0)


    ;;-------------------------------------------------------------------------
    ;; perl-pod-gt-double

    (mapcar (lambda (elem)
              (let ((str  (car elem))
                    (want (cadr elem))
                    (name (car (cddr elem))))
                (erase-buffer)
                (insert str "\n\n" (or name "") "\n")
                (goto-char (+ 2 (point-min)))
                (perl-pod-gt-double)
                (let ((got (buffer-substring (point-min) (line-end-position))))
                  (unless (equal got want)
                    (error "perl-pod-gt-double: got %s want %s" got want))))
              (setq count (1+ count)))

            '(
              ("C<xE<gt>E<gt>y>" "C<< xE<gt>E<gt>y >>"
               "... run multiple gt not converted")

              ("C<>"        "C<<  >>")
              ("C<x>"       "C<< x >>"   "x unchanged")
              ("C<x<y>"     "C<< x<y >>" "x<y unchanged")

              ("C<xE<lt>y>" "C<< x<y >>"
               "... single lt converted")

              ("C<XE<lt>>" "C<< XE<lt> >>"
               "... E<lt> not changed to X<> markup")

              ;; E<gt> not changed after E<lt> as E<lt>> would be unclear
              ("C<XE<lt>E<gt>>" "C<< XE<lt>E<gt> >>")

              ;; E<gt> not changed after E<sol> as E<sol>> would be unclear
              ("C<blah E<sol>E<gt> blah>" "C<< blah E<sol>E<gt> blah >>")

              ("C<xE<gt>y>" "C<< x>y >>"
               "... single gt converted")

              ("C<E<lt>E<gt>>" "C<< <> >>"
               "... adjacent lt/gt converted")))


    ;;-------------------------------------------------------------------------
    ;; perl-pod-gt-nobreak-p

    (mapcar (lambda (elem)
              (let ((str  (car elem))
                    (want (cadr elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (search-forward "|")
                (delete-char -1)
                (let ((got (if (perl-pod-gt-nobreak-p) t nil)))
                  (unless (equal got want)
                    (error "perl-pod-gt-nobreak-p: got %s want %s" got want))))
              (setq count (1+ count)))

            '(("C<< C<B| >>"     t)
              ("C<< C<B>| >>"    t)
              ("C<< I<x | y> >>" nil)
              ("fdjsk jfksd C<< S<x | y> >> fjsdk fjksd >>" nil)
              ("fdjsk jfksd S<< S<x y> >> fjsdk | fjksd >>" nil)
              ("fdjsk jfksd S<< C<x |y> >> fjsdk fjksd >>" t)
              ("fdjsk jfksd s<< C<x |y> >> fjsdk fjksd >>" nil)

              ("fdjsk S<<< C<< B<x> >> |>>> jfkds" t)
              ("fdjsk S<<< C<< B<x> |>> >>> jfkds" t)
              ("fdjsk S<<< C<< B<x |> >> >>> jfkds" t)
              ("fdjsk S<<< C<< x >> C << y >> | x >>> jfkds" t)
              ("fdjsk S< E<x | y> > jfkds" t)
              ("S<< x y >> C<< x |y >>" nil)
              ("X<< x y >> C<< x |y >>" nil)
              ("C<< x y >> S<< x |y >>" t)
              ("C<< x y >> X<< x |y >>" t)
              ("C< |foo"     t)
              ("C<< |foo"    t)
              ("C<< foo |>>" t)
              ("C<< foo bar |>>" t)
              ("C<< foo |bar >>" nil)
              ("C<< foo bar |>>" t)
              ("S<< foo bar |>>" t)
              ("X<< foo bar |>>" t)
              ("S<< foo |bar >>" t)
              ("S<< foo E<x |bar> >>" t)
              ("S<< foo C<x |bar> >>" t)
              ("C<< foo S<T |bar" nil)

              ("C<!> |bar" t)
              ("S<!> |bar" nil)
              ("c<!> |bar" nil)))


    ;;-------------------------------------------------------------------------
    ;; perl-pod-gt-warn-enable -- buffer-local after-change-functions

    (require 'perl-pod-gt)
    (setq after-change-functions nil)
    (let ((want after-change-functions))
      (with-temp-buffer
        (perl-pod-gt-warn-enable))
      (let ((got after-change-functions))
        (unless (equal got want)
          (error "perl-pod-gt-warn-enable changes global after-change-functions: got %S want %S" got want)))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; perl-pod-gt-single

    (mapcar (lambda (elem)
              (let ((str  (car elem))
                    (want (cadr elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ 2 (point-min)))

                (condition-case err
                    (perl-pod-gt-single)
                  (error
                   (erase-buffer)
                   (insert (error-message-string err))))

                (let ((got (buffer-string)))
                  (unless (equal got want)
                    (error "perl-pod-gt-single: got %s want %s" got want))))
              (setq count (1+ count)))

            '(("C<<  >>"      "C<>")
              ("C<< x >>"     "C<x>")
              ("F<< x<y >>"   "F<xE<lt>y>")
              ("I<< x>y >>"   "I<xE<gt>y>")
              ("S<<< x>y >>>" "S<xE<gt>y>")
              ("B<<<< x >>>>"  "B<x>")

              ("C<< E<lt> >>"      "C<EE<lt>ltE<gt>>")
              ("C<< E<gt> >>"      "C<EE<lt>gtE<gt>>")
              ("C<< E<lt>E<gt> >>" "C<EE<lt>ltE<gt>EE<lt>gtE<gt>>")

              ("c<< x>y >>"   "Not in a C<<...>> etc form")
              ("C<abc>"       "Not in a C<<...>> etc form")))


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))

