;; Copyright 2016, 2017, 2019 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

;; XEmacs 21.4 comes with auctex which rudely overrides plain-tex-mode to
;; its TeX-plain-tex-mode but that mode doesn't work under -batch it seems
;; (error out with Wrong type argument: number-char-or-marker-p, nil.
;; Loading tex-mode.el gets the ordinary plain-tex-mode which is ok.
;;
(require 'tex-mode)

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-tex-increase--temp*")
        (verbose nil))
    (switch-to-buffer tempbuf)

    (when verbose
      (message "plain-tex-mode is %S" (symbol-function 'plain-tex-mode)))

    ;; (message "featurep tex-mode %S" (featurep 'tex-mode))
    ;; (message "featurep tex-site %S" (featurep 'tex-site))
    ;; (message "plain-tex-mode %S" (symbol-function 'plain-tex-mode))
    ;;
    ;; (message "featurep tex-mode %S" (featurep 'tex-mode))
    ;; (message "featurep tex-site %S" (featurep 'tex-site))
    ;; (message "plain-tex-mode %S" (symbol-function 'plain-tex-mode))
    ;;
    ;; (let (elt
    ;;       (list TeX-mode-alist) elt)
    ;;   (while list
    ;;     (setq elt (car (pop list)))
    ;;     (message "%S %S" elt (get elt 'tex-saved))))
    ;;
    ;; (kill-emacs)
    ;;
    ;; (condition-case err
    ;;     (plain-tex-mode)
    ;;   (error (backtrace)
    ;;          (format "error: %s"
    ;;                  (error-message-string err))))
    ;;
    ;; (when verbose (message "TeX-modes now %S"
    ;;                        (if (boundp 'TeX-modes)
    ;;                            TeX-modes "[not bound]")))
    ;; (message "tex-saved %S" (get 'plain-tex-mode 'tex-saved))
    ;; (message "tex-saved %S" (get 'TeX-plain-tex-mode 'tex-saved))

    ;;  (erase-buffer)
    ;; (funcall 'plain-tex-mode)

    ;; ;    (setq debug-on-error t)
    ;; ;    (plain-tex-mode)
    ;; ;    (quit)
    ;; ;    (when (fboundp 'TeX-modes-set)
    ;; ;      (message "TeX-modes-set to %S" (remove 'plain-tex-mode TeX-modes))
    ;; ;      (TeX-modes-set 'TeX-modes-set (remove 'plain-tex-mode TeX-modes)))
    ;;

    ;;-------------------------------------------------------------------------
    ;; autoload

    (when verbose (message "autoloads"))
    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(tex-increase tex-decrease))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; tex-increase

    (mapcar (lambda (elem)
              (when verbose (message "elem %S" elem))
              (let ((arg   (nth 0 elem))
                    (pos   (nth 1 elem))
                    (input (nth 2 elem))
                    (want  (nth 3 elem))
                    (mode  (or (nth 4 elem) 'tex-mode)))
                (erase-buffer)
                (when verbose (message "mode %S" mode))
                (funcall mode)
                (when verbose (message "input %S" input))
                (insert input)
                (goto-char (+ (point-min) pos))
                ;; (end-of-line)
                (let ((got (condition-case err
                               (progn
                                 (tex-increase arg)
                                 (buffer-string))
                             (error (format "error: %s"
                                            (error-message-string err))))))
                  (unless (equal want got)
                    (when verbose (message "bad"))
                    (goto-char (point-max))
                    (insert "\n\n")
                    (insert (format "arg %S, mode %S %S\n" arg mode major-mode))
                    (insert "---input---\n")
                    (insert input)
                    (insert "\n---got---\n")
                    (insert (format "%s\n" got))
                    (insert "\n---want---\n")
                    (insert want)
                    (error "wrong at %s\n%s\n" count (buffer-string)))))
              (setq count (1+ count)))

            '(
              ;; ARG POS INPUT WANT MODE
              (  1 5 "\\bigl\\lvert" "\\Bigl\\lvert" )
              (  1 5 "\\bigr\\rvert" "\\Bigr\\rvert" )
              (  1 5 "\\bigr\\rparen" "\\Bigr\\rparen" )
              (  1 5 "\\bigr)" "\\Bigr)" )

              (  1 0 "\\rvert" "\\bigr\\rvert")
              ( -1 0 "\\bigr\\rvert" "\\rvert")
              (  1 0 "\\bigr\\rvert" "\\Bigr\\rvert" )
              ( -1 0 "\\Bigr\\rvert" "\\bigr\\rvert" )

              ( 1 0 "\\," "\\>" plain-tex-mode)
              (-1 0 "\\;" "\\>" plain-tex-mode)
              ( 1 0 "\\," "\\:" latex-mode)
              (-1 0 "\\;" "\\:" latex-mode)

              (-1 0 "\\Big\\sum" "\\big\\sum")
              ( 1 0 "(" "\\bigl(")
              (-1 0 "(" "error: Nothing more for (")
              ( 1 0 "[" "\\bigl[")
              ( 1 0 "\\big\\sum" "\\Big\\sum")
              (-1 0 "\\big\\sum" "\\sum")

              ( 1 0 "\\{" "\\bigl\\{")
              ( 2 0 "\\}" "\\Bigr\\}")
              (-1 0 "\\bigr\\}" "\\}")
              ( 1 0 "\\lbrace" "\\bigl\\lbrace")

              (1 0 "\\scriptscriptstyle" "\\scriptstyle")
              (1 0 "\\scriptstyle" "\\textstyle")
              (1 0 "\\textstyle" "\\displaystyle")

              (1 1 "((" "(\\bigl(")
              (1 1 "(\\medskip" "(\\bigskip")
              (1 8 "\\medskip(" "\\medskip\\bigl(")
              (1 8 "\\medskip\\medskip" "\\medskip\\bigskip")

              ))


;;-------------------------------------------------------------------------
;; unload-feature

(when verbose (message "unload-feature"))

;; couple of cycles
(require 'tex-increase)
(unload-feature 'tex-increase t)
(require 'tex-increase)
(unload-feature 'tex-increase t)
(setq count (1+ count))


;;-------------------------------------------------------------------------

(bury-buffer)
(message "  %d tests" count)
count))
